/******************************************************************************\
 * addfoldericons: ConcurrentPathProcessor.m
 *
 * Derive a class from NSOperation which can be used to concurrently process a
 * full POSIX path to a folder in order to update that folder's icon. The class
 * may be run by, for example, adding it as an operation to an NSOperationQueue
 * instance.
 *
 * (C) Hipposoft 2009 <ahodgkin@rowing.org.uk>
 \******************************************************************************/

#include <stdio.h>

/* Application functions and data */

#import "ConcurrentPathProcessor.h"
#import "GlobalConstants.h"

/* Library functions and data */

#import "GlobalSemaphore.h"
#import "Icons.h"
#import "ThumbnailGenerator.h"

@implementation ConcurrentPathProcessor

/******************************************************************************\
 * - initWithPathAndBackground
 *
 * Initialise the NSOperation derivative class. Local copies are taken of all
 * objects given in the input parameters so the caller can discard its own
 * copy immediately after calling if it so wishes.
 *
 * The global semaphore system must be initialised before calling here. See
 * "globalSemaphoreInit" in "GlobalSemaphore.[h|m]".
 *
 * In:  Pointer to an NSString giving the full POSIX path to the folder of
 *      interest - this folder will be given a custom icon if the object's
 *      "main" method gets invoked;
 *
 *      CGImageRef giving a background image - for more information, see the
 *      documentation in the library code for the "backgroundImage" parameter
 *      of "generateThumbnailForFolder" in "ThumbnailGenerator.[h|m]".
 *
 * Out: self.
\******************************************************************************/

-( id ) initWithPathAndBackground : ( NSString * ) fullPosixPath
                                  : ( CGImageRef ) backgroundImage
{
    if ( self = [ super init ] )
    {
        pathData      = [ fullPosixPath retain ];
        backgroundRef = CGImageCreateCopy( backgroundImage );
    }
    
    return self;
}

/******************************************************************************\
 * - initWithpath
 *
 * Release any and all resources claimed during the object's lifecycle. Usually
 * invoked only by the Grand Central Dispatch mechanism's Cocoa code.
\******************************************************************************/

-( void ) dealloc
{
    CFRelease( backgroundRef );
    [ pathData release ];
    [ super dealloc ];
}

/******************************************************************************\
 * - main
 *
 * Main processing loop. Usually invoked only by the Grand Central Dispatch
 * mechanism's Cocoa code. The folder given in "initWithPathAndBackground"
 * will have gained an updated icon on exit provided there were no errors.
\******************************************************************************/

-( void ) main
{
    @try
    {
        /* Generate the thumbnail */
        
        OSStatus   status     = noErr;
        CGImageRef finalImage = generateThumbnailForFolder
        (
            pathData,
            NO,
            SKIP_PACKAGES,
            backgroundRef,
            &status
        );

        if ( finalImage )
        {
            /* Apply the thumbnail to the folder */
            
            IconFamilyHandle iconHnd = NULL;
            status = createIconFamilyFromCGImage( finalImage, &iconHnd );
            
            if ( status == noErr && iconHnd != NULL )
            {
                globalSemaphoreClaim();
                status = saveCustomIcon( pathData, iconHnd );
                globalSemaphoreRelease();

                DisposeHandle( ( Handle ) iconHnd );
            }
            
            CFRelease( finalImage );
        }
        
        if ( status != noErr )
        {
            NSLog
            (
                @"%@: Failed for '%@' with OSStatus code %d (&%04X) and errno value %d (&%04X): %@",
                [ NSString stringWithUTF8String: PROGRAM_STRING ],
                pathData,
                ( int          ) status,
                ( unsigned int ) status,
                ( int          ) errno,
                ( unsigned int ) errno,
                [ NSString stringWithUTF8String: strerror( errno ) ]
            );
            
            globalSemaphoreClaim();
            globalErrorFlag = YES;
            globalSemaphoreRelease();
        }
    }
    @catch ( NSException * exception )
    {
        NSLog
        (
            @"%@: Exception '%@': %@",
            [ NSString stringWithUTF8String: PROGRAM_STRING ],
            [ exception name   ],
            [ exception reason ]
        );

        globalSemaphoreClaim();
        globalErrorFlag = YES;
        globalSemaphoreRelease();
    }
}

@end /* @implementation ConcurrentPathProcessor */
